﻿using System;
using System.Collections.Generic;
#if MONOTOUCH || NET || MONODROID
using System.Web;
#endif
#if NETFX_CORE ||WINDOWS_PHONE
using System.Net;
#endif
namespace WolframAlpha.WrapperCore
{
    /// <summary>
    /// Query to Wolfram Alpha
    /// </summary>
    public partial class WolframAlphaQuery
    {
        #region Constants
        /// <summary>
        /// Base address url
        /// </summary>
        private const string Baseadress = "http://api.wolframalpha.com/v2/query?";
        #endregion

        #region Fields

        /// <summary>
        /// Application Key
        /// </summary>
        public string APIKey
        {
            get;
            set;
        }

        /// <summary>
        /// List of answer format like HTML or PlainText
        /// </summary>
        public List<string> Formats
        {
            get;
            set;
        }

        /// <summary>
        /// Query string
        /// </summary>
        public string Query
        {
            get;
            set;
        }

        /// <summary>
        /// Time Limit in Millesecond
        /// </summary>
        public int TimeLimit
        {
            get;
            set;
        }

        /// <summary>
        /// Allow of Disable caching answer result
        /// </summary>
        public bool AllowCached
        {
            get;
            set;
        }

        /// <summary>
        /// Is this operation Async
        /// </summary>
        public bool Asynchronous
        {
            get;
            set;
        }

        /// <summary>
        /// 
        /// </summary>
        public bool MoreOutput
        {
            get;
            set;
        }
        /// <summary>
        /// Substitution
        /// </summary>
        private string _substitution;
        /// <summary>
        /// Assumption
        /// </summary>
        private string _assumption;
        /// <summary>
        /// Pod Title
        /// </summary>
        private string _podTitle;
        #endregion

        #region Properties

        /// <summary>
        ///  Substitutions array
        /// </summary>
        public string[] Substitutions
        {
            get
            {
                return _substitution.Split(new[] { "&substitution=" }, StringSplitOptions.RemoveEmptyEntries);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public string[] Assumptions
        {
            get
            {
                return _assumption.Split(new[] { "&assumption=" }, StringSplitOptions.RemoveEmptyEntries);
            }
        }

        /// <summary>
        /// Podtitles array
        /// </summary>
        public string[] PodTitles
        {
            get
            {
                return _podTitle.Split(new[] { "&assumption=" }, StringSplitOptions.RemoveEmptyEntries);
            }
        }

        /// <summary>
        /// Generate WAQuery string from our search string
        /// </summary>
        public string FullQueryString
        {
            get
            {
                string returnstring = Baseadress;
                returnstring += "input=" + Query;
                if (Formats != null && Formats.Count != 0)
                {
                    returnstring += "&format=";
                    for (int i = 0; i < Formats.Count - 1; i++)
                    {
                        returnstring += Formats[i] + ",";
                    }
                    returnstring += Formats[Formats.Count - 1];
                }
                if (_assumption != null && Assumptions.Length != 0)
                {
                    returnstring += "&assumption=" + Assumptions[0];
                }
                return returnstring + "&appid=" + APIKey;
            }
        }
        #endregion

        #region Constructors
        /// <summary>
        /// Default Constructor
        /// </summary>
        public WolframAlphaQuery()
        {
            Formats = new List<string>();
            APIKey = string.Empty;
            Query = string.Empty;
            _podTitle = string.Empty;
            _assumption = string.Empty;
            _substitution = string.Empty;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Add Pod Title to Query
        /// </summary>
        /// <param name="podTitle">Pod Title</param>
        /// <param name="checkForDuplicates">Check for duplicates this parameter</param>
        public void AddPodTitle(string podTitle, bool checkForDuplicates = false)
        {
            if (checkForDuplicates == true && _podTitle.Contains("&PodTitle=" + encode(podTitle)))
                return;
            _podTitle += @"&podtitle=" + encode(podTitle);
        }

        /// <summary>
        /// Add Substitution to Query 
        /// </summary>
        /// <param name="substitution">Substitution</param>
        /// <param name="checkForDuplicates">Check for duplicates this parameter</param>
        public void AddSubstitution(string substitution, bool checkForDuplicates = false)
        {
            if (checkForDuplicates == true && _substitution.Contains("&substitution=" + encode(substitution)))
                return;
            _substitution += "&substitution=" + encode(substitution);
        }

        /// <summary>
        /// Add Assumption to Query 
        /// </summary>
        /// <param name="assumption">Assumption string</param>
        /// <param name="checkForDuplicates">Check for duplicates this parameter</param>
        public void AddAssumption(string assumption, bool checkForDuplicates = false)
        {
            if (checkForDuplicates == true && _assumption.Contains("&substitution=" + encode(assumption)))
                return;
            _assumption += "&assumption=" + encode(assumption);
        }

        /// <summary>
        /// Add Assumption to Query 
        /// </summary>
        /// <param name="assumption">Assumption WolframAlphaAssumption</param>
        /// <param name="checkForDuplicates">Check for duplicates this parameter</param>
        public void AddAssumption(WolframAlphaAssumption assumption, bool checkForDuplicates = false)
        {
            if (assumption == null || (checkForDuplicates == true && _assumption.Contains("&substitution=" + encode(assumption.Word)) || string.IsNullOrEmpty(assumption.Word)))
                return;
            _assumption += "&assumption=" + encode(assumption.Word);
        }
        private static string encode(string encodeString)
        {
#if MONOTOUCH || NET || MONODROID || WINDOWS_PHONE
            return HttpUtility.UrlEncode(encodeString);
#endif
#if NETFX_CORE            
            return WebUtility.HtmlEncode(encodeString);
#endif
        }
        #endregion
    }
}
